#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE', which is part of this source code package.
#
from django.utils import timezone
from . import settings
from .models import Credentials
from datetime import timedelta
import requests
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def replyify_token_is_valid(user):
    try:
        return Credentials.objects.get(user=user).is_valid()
    except Credentials.DoesNotExist:
        return False


def refresh_access_token(user):
    creds = Credentials.objects.get(user=user)

    data = {
        'grant_type': 'refresh_token',
        'client_id': settings.REPLYIFY_CLIENT_ID,
        'client_secret': settings.REPLYIFY_CLIENT_SECRET,
        'refresh_token': creds.refresh_token
    }

    url = settings.REPLYIFY_TOKEN_URL
    logger.info('** REPLYIFY: Refresh Token URL - {}'.format(url))
    logger.info('** REPLYIFY: Token data - {}'.format(data))
    response = requests.post(url=url, data=data)
    user.replyify_credentials = store_credentials(user, response.json())
    return user


def store_credentials(user, replyify_json=None):
    assert user is not None
    creds, _ = Credentials.objects.get_or_create(user=user)
    creds.access_token = replyify_json['access_token']
    creds.refresh_token = replyify_json['refresh_token']
    creds.expires = timezone.now() + timedelta(seconds=replyify_json['expires_in'])
    creds.scope = replyify_json['scope']
    creds.token_type = replyify_json['token_type']
    creds.save()
    return creds



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE', which is part of this source code package.
#
from django.contrib import admin
from django.contrib.auth import get_user_model
from . import settings
from .models import Credentials


def user_search_fields():
    User = get_user_model()
    USERNAME_FIELD = getattr(User, 'USERNAME_FIELD', None)
    fields = []
    if USERNAME_FIELD is not None:
        # Using a Django 1.5+ User model
        fields = [
            'user__{}'.format(USERNAME_FIELD),
            'user__{}'.format(settings.REPLYIFY_USER_ID_FIELD)
        ]
    return fields


class CredentialsAdmin(admin.ModelAdmin):
    search_fields = ['access_token'] + user_search_fields()
    raw_id_fields = ('user',)


admin.site.register(Credentials, CredentialsAdmin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE', which is part of this source code package.
#
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse
from django.shortcuts import redirect
from django.contrib import messages
from .models import Credentials
from .utils import store_credentials, refresh_access_token
from . import settings
import requests
import urllib
import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def index(request=None):
    return HttpResponse('Replyify-OAuth2 index.')


@login_required
def authorize(request=None):
    # uid = getattr(request.user, settings.REPLYIFY_USER_ID_FIELD)
    next_url = request.GET.get('next', request.GET.get('state', '/'))
    if request.GET.get('denied_redirect'):
        request.session['replyify-denied-redirect'] = request.GET['denied_redirect']
    logger.info('** REPLYIFY: /authorize - Next URL: {}'.format(next_url))
    # state = get_random_string(20, 'abcdefghijklmnopqrstuvwxyz0123456789')
    request.session['state'] = next_url
    params = {
        'client_id': settings.REPLYIFY_CLIENT_ID,
        'redirect_uri': settings.REPLYIFY_REDIRECT_URI,
        'response_type': 'code',
        'state': next_url,
    }
    url = '{0}?{1}'.format(settings.REPLYIFY_AUTH_URL, urllib.urlencode(params))
    logger.info('** REPLYIFY: /authorize - Redirecting to: {}'.format(url))
    return redirect(url)


@login_required
def callback(request=None):
    if request.GET.get('error'):
        logger.info('** REPLYIFY: /callback - Access Denied!')
        messages.error(request, request.GET.get('error').replace('_', ' ').title())
        next_url = request.session.get('replyify-denied-redirect', settings.REPLYIFY_DENIED_REDIRECT)
        logger.info('** REPLYIFY: /callback - Denied Redirect: {}'.format(next_url))
        return redirect(next_url)
    logger.info('** REPLYIFY: /callback')
    data = {
        'grant_type': 'authorization_code',
        'code': request.GET['code'],
        'client_id': settings.REPLYIFY_CLIENT_ID,
        'client_secret': settings.REPLYIFY_CLIENT_SECRET,
        'redirect_uri': settings.REPLYIFY_REDIRECT_URI
    }
    url = settings.REPLYIFY_TOKEN_URL
    logger.info('** REPLYIFY: Token URL - {}'.format(url))
    logger.info('** REPLYIFY: Token data - {}'.format(data))
    response = requests.post(url=url, data=data)
    response_data = response.json()
    logger.info('** REPLYIFY: Response data - {}'.format(response_data))
    if response_data.get('error'):
        logger.error('** REPLYIFY ERROR: {}'.format(response_data['error']))
        messages.error(request, 'REPLYIFY ERROR: {}'.format(response_data['error']))
    else:
        store_credentials(request.user, response_data)
    return redirect(request.GET.get('state', '/'))


@login_required
def refresh(request=None):
    if 'error' in request.GET:
        raise Exception(request.GET['error'])
    logger.info('** REPLYIFY: /refresh')
    try:
        next_url = request.GET.get('next', request.GET.get('state', '/'))
        request.user = refresh_access_token(request.user)
        return redirect(next_url)
    except Credentials.DoesNotExist:
        authorize(request)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE', which is part of this source code package.
#
from django.conf import settings

REPLYIFY_CLIENT_ID = getattr(settings, 'REPLYIFY_CLIENT_ID', None)
REPLYIFY_CLIENT_SECRET = getattr(settings, 'REPLYIFY_CLIENT_SECRET', None)
REPLYIFY_REDIRECT_URI = getattr(settings, 'REPLYIFY_REDIRECT_URI', None)

REPLYIFY_AUTH_URL = getattr(settings, 'REPLYIFY_AUTH_URL', 'https://app.replyify.com/oauth2/authorize')
REPLYIFY_TOKEN_URL = getattr(settings, 'REPLYIFY_TOKEN_URL', 'https://app.replyify.com/oauth2/token')

REPLYIFY_USER_ID_FIELD = getattr(settings, 'REPLYIFY_USER_ID_FIELD', 'id')
REPLYIFY_DENIED_REDIRECT = getattr(settings, 'REPLYIFY_DENIED_REDIRECT', '/')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE', which is part of this source code package.
#
from __future__ import unicode_literals
from django.conf import settings
from django.db import models
from django.utils import timezone


class Credentials(models.Model):
    user = models.OneToOneField(getattr(settings, 'AUTH_USER_MODEL', 'auth.User'), on_delete=models.CASCADE, related_name='replyify_credentials')
    access_token = models.CharField(max_length=50)
    refresh_token = models.CharField(max_length=50)
    expires = models.DateTimeField(default=timezone.now)
    scope = models.CharField(max_length=50)
    token_type = models.CharField(max_length=50)
    updated = models.DateTimeField(auto_now=True)
    created = models.DateTimeField(auto_now_add=True)

    def is_expired(self):
        if timezone.now() > self.expires:
            return True
        return False

    def is_valid(self):
        return not self.is_expired()

    def __unicode__(self):
        return '<Replyify Creds: {}>'.format(self.access_token)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE', which is part of this source code package.
#
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Credentials',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('access_token', models.CharField(max_length=50)),
                ('refresh_token', models.CharField(max_length=50)),
                ('expires', models.DateTimeField(default=django.utils.timezone.now)),
                ('scope', models.CharField(max_length=50)),
                ('token_type', models.CharField(max_length=50)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='replyify_credentials', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE', which is part of this source code package.
#
from functools import wraps
from django.core.urlresolvers import reverse
from django.shortcuts import redirect
from .models import Credentials
from .utils import refresh_access_token
import traceback


def replyify_auth_required(func):
    def decorator(func):
        def inner_decorator(request, *args, **kwargs):
            try:
                if request.user.replyify_credentials.is_valid():
                    return func(request, *args, **kwargs)
                try:
                    request.user = refresh_access_token(request.user)
                    if request.user.replyify_credentials.is_valid():
                        return func(request, *args, **kwargs)
                except Exception:
                    traceback.print_exc()
            except Credentials.DoesNotExist:
                return redirect(reverse('replyify:authorize') + '?next=' + request.GET.get('next', request.get_full_path()))
            return redirect(reverse('replyify:refresh') + '?next=' + request.GET.get('next', request.get_full_path()))
        return wraps(func)(inner_decorator)
    return decorator(func)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE', which is part of this source code package.
#

import os
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

# See https://github.com/pybuilder/pybuilder/issues/56
del os.link


with open(os.path.join(os.path.dirname(__file__), 'VERSION'), 'r') as v:
    VERSION = v.read()


with open(os.path.join(os.path.dirname(__file__), 'README.rst'), 'r') as readme:
    README = readme.read()


setup(
    name='django-replyify-oauth2',
    version=VERSION,
    description='Replyify OAuth2 consumer for Django',
    long_description=README,
    author='Replyify',
    author_email='team@replyify.com',
    url='http://replyify.com',
    keywords=[
        'replyify',
        'oauth2',
        'django'
    ],
    packages=[
        'replyify_oauth2',
        'replyify_oauth2.migrations',
    ],
    install_requires=[
        'Django>=1.6',
        'requests>=0.8.8',
    ],
    scripts=[],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.6',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE', which is part of this source code package.
#
from __future__ import unicode_literals
from django import VERSION as DJANGO_VERSION

if DJANGO_VERSION >= (1, 7):
    from django.apps import AppConfig

    class ReplyifyOAuth2Config(AppConfig):
        name = 'replyify_oauth2'
        verbose_name = 'Replyify OAuth2'
        label = 'replyify_oauth2'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE', which is part of this source code package.
#
from django import VERSION as DJANGO_VERSION
if DJANGO_VERSION >= (1, 7):
    default_app_config = 'replyify_oauth2.apps.ReplyifyOAuth2Config'

__version__ = '0.0.5'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE', which is part of this source code package.
#
from django.conf.urls import url
from . import views

urlpatterns = [
    url(r'^authorize/', views.authorize, name='authorize'),
    url(r'^callback/', views.callback, name='callback'),
    url(r'^refresh/', views.refresh, name='refresh'),
    url(r'^$', views.index, name='index'),
]
